#!/usr/bin/env python
# pylint: disable-msg=W0142,R0903
"""
_StoreFail_

Util class to stage out a set of files from a job report to the /store/unmerged/fail
namespace

"""
__version__ = "$Revision: 1.3 $"
__revision__ = "$Id: StoreFail.py,v 1.3 2009/06/26 15:02:17 direyes Exp $"

from StageOut.StageOutError import StageOutFailure
from StageOut.StageOutMgr import StageOutMgr


def modifyLFN(inputLfn):
    """
    _modifyLFN_

    Util to tweak a normal LFN to be a /store/unmerged/fail LFN
    Simple algorithm to start with, split of /store/whatever and replace
    it with /store/unmerged/fail/whatever

    """
    if not inputLfn.startswith('/store/'):
        return None        
    lfnSplit = [ x for x in inputLfn.split("/") if len(x) != 0 ]
    newlfnList = []
    newlfnList.append(lfnSplit[0])
    newlfnList.append("unmerged/fail")
    newlfnList.extend(lfnSplit[1:])
    newLfn = "/%s" % "/".join(newlfnList)
    return newLfn


class StoreFailMgr:
    """
    _StoreFailMgr_

    """
    def __init__(self, jobReport):
        self.report = jobReport
        self.mgr = StageOutMgr()
        self.failures = {}




    def __call__(self):
        """
        _operator()_

        Invoke stage out of files to /store/unmerged/fail based on
        information in the report provided.

        Generate a list of the LFNs that were staged out as a
        return value

        """
        stagedOutFiles = []
        for fileToStage in self.report.files:
            lfn = fileToStage['LFN']
            newLfn = modifyLFN(lfn)
            if newLfn is None:
                msg = "LFN: %s is either corrupted or " % lfn
                msg += "the file should not be staged out. Skipping it."
                print msg
                continue
            print "Remapping LFN:\n%s\n" % lfn
            print " -TO-\n%s\n" % newLfn
            print "Staging Out: %s" % newLfn
            fileToStage['LFN'] = newLfn
            try:
                result = self.mgr(**fileToStage)
                fileToStage.update(result)
                stagedOutFiles.append(newLfn)
            except StageOutFailure, ex:
                msg = "Unable to stage out %s " % newLfn
                msg += " due to Stage Out Failure:\n"
                msg += str(ex)
                self.failures[newLfn] = msg
                continue

        return stagedOutFiles








